package fmon.util

import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory
import com.fasterxml.jackson.module.scala.DefaultScalaModule
import com.fasterxml.jackson.module.scala.experimental.ScalaObjectMapper

import java.io.InputStream

import scala.io.Source

import fmon.util._

object YamlHelper {
  val mapper = new ObjectMapper(new YAMLFactory()) with ScalaObjectMapper
  mapper.registerModule(DefaultScalaModule)
  mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
  
  def extract[T](text : String)(implicit m : Manifest[T]) : T = {
    mapper.readValue[T](text)
  }
  
  def extract[T](source : Source)(implicit mf : Manifest[T]) : T = {
    val text = source.getLines().mkString("\n")
    extract(text)
  }
  
  def extract[T](source : InputStream)(implicit m : Manifest[T]) : T = {
    mapper.readValue[T](source)
  }
  
  def extractSeq[T](text : String)(implicit m : Manifest[T]) : IndexedSeq[T] = {
    mapper.readValue[IndexedSeq[T]](text)
  }
  
  def extractSeq[T](source : Source)(implicit mf : Manifest[T]) : IndexedSeq[T] = {
    val text = source.getLines().mkString("\n")
    extractSeq(text)
  }
  
  def extractSeq[T](source : InputStream)(implicit m : Manifest[T]) : IndexedSeq[T] = {
    mapper.readValue[IndexedSeq[T]](source)
  }
}